@@ -32,6 +32,7 @@
 #include "gok-modifier.h"
 #include "gok-scanner.h"
 #include "gok-feedback.h"
+#include "gok-word-complete.h"
 #include "gok-data.h"
 #include "gok-branchback-stack.h"
 #include "gok-modifier-keymasks.h"
@@ -378,7 +379,7 @@ gok_key_from_xkb_key (GokKey *prevKey, GokKeyboard *pKeyboard, Display *display,
   pKey = gok_key_new (prevKey, NULL, pKeyboard);
   numlock_mask = gok_key_get_numlock_mask (display);
   is_modifier = FALSE;
-  pKey->has_text = TRUE; /* TODO: handle non-textual keyboard keys! */
+  pKey->has_text = TRUE; /* TODO: handle non-textual keyboard key caps! */
   pKey->Top = row;
   pBounds = &pGeom->shapes[keyp->shape_ndx].bounds;
   pKey->Bottom = row + MAX (1, (pBounds->y2 - pBounds->y1)/(pBounds->x2 - pBounds->x1));
@@ -620,6 +621,7 @@ void gok_key_delete (GokKey* pKey, GokKeyboard* pKeyboard, gboolean bDeleteButto
 
 	gok_feedback_drop_refs (pKey);
 	gok_scanner_drop_refs (pKey);
+	gok_repeat_drop_refs (pKey);
 }
 
 /**
@@ -803,7 +805,11 @@ gboolean gok_key_initialize (GokKey* pKey, xmlNode* pNode)
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"repeat-next") == 0)
 	{
 		pKey->Type = KEYTYPE_REPEATNEXT;
-		pKey->Style = KEYSTYLE_NORMAL;
+		pKey->Style = KEYSTYLE_REPEATNEXT;
+		pKey->has_image = TRUE;
+		pKey->pImage = gok_keyimage_new (pKey, NULL);
+		pKey->pImage->type = IMAGE_TYPE_INDICATOR;
+		pKey->ComponentState.active = FALSE;
 	}
 	else if (xmlStrcmp (pStringAttributeValue, (const xmlChar *)"mouse") == 0)
 	{
@@ -1589,10 +1595,15 @@ void gok_key_set_button_name (GokKey* pKey)
 		strcpy (styleText, "StyleTextNormal");
 		break;
 
-	case KEYSTYLE_BRANCHGUIACTIONS:
 	case KEYSTYLE_PAGESELECTION:
+		strcpy (styleButton, "StyleButtonBranchPageSelection");
+		strcpy (styleText, "StyleTextNormal");
+		break;
+
+	case KEYSTYLE_BRANCHGUIACTIONS:
 	case KEYSTYLE_HYPERLINK:
 	case KEYSTYLE_EDIT:
+	case KEYSTYLE_REPEATNEXT:
 		strcpy (styleButton, "StyleButtonBranchGuiActions");
 		strcpy (styleText, "StyleTextNormal");
 		break;
@@ -1648,7 +1659,7 @@ void gok_key_set_button_name (GokKey* pKey)
 void
 gok_key_update_toggle_state (GokKey *pKey)
 {
-	if (pKey && GTK_IS_TOGGLE_BUTTON (pKey->pButton)) {
+	if (pKey && pKey->pButton && GTK_IS_TOGGLE_BUTTON (pKey->pButton)) {
 	        gboolean is_active = FALSE; 
 		if (pKey->ModifierName != NULL)
 			is_active = 
@@ -2204,3 +2215,59 @@ gok_key_isRepeatable(GokKey* pKey)
 	gok_log_leave();
 	return returncode;
 }
+
+
+static GokOutput *last_wordcomplete_output = NULL;
+
+/**
+* gok_key_wordcomplete_output
+* 
+* Gets the output for a word prediction key.
+*
+* @pKey: Pointer to the word completion key that will be output.
+*
+* returns: A pointer to the output.
+**/
+GokOutput* 
+gok_key_wordcomplete_output (GokKey *pKey, GokWordComplete *complete)
+{
+	gchar* LabelText;
+	gchar *word_part = gok_wordcomplete_get_word_part (complete);
+	g_assert (pKey != NULL);
+
+	/* make sure this is a word completion key */
+	if (pKey->Type != KEYTYPE_WORDCOMPLETE)
+	{
+		gok_log_x ("Hey, this is not a word completion key!\n");
+		return NULL;
+	}
+	
+	/* subtract the part word from the output */
+	LabelText = gok_key_get_label (pKey);
+	if (LabelText == NULL)
+	{
+		return NULL;
+	}
+	
+	if (word_part && (g_utf8_strlen (word_part, -1) < g_utf8_strlen (LabelText, -1)))
+	{
+		gchar *cp;
+                /* 
+		 * note that this won't match if the decompositions don't match. Should we
+		 * decompose to canonical first ? 
+		 */ 
+		cp = strstr (LabelText, word_part); 
+		/* sanity check: is our word_part found in our label? */
+		if (cp) {
+			gint offset = strlen (word_part); /* in bytes, not chars */
+			if (last_wordcomplete_output) gok_output_delete_all (last_wordcomplete_output);
+			last_wordcomplete_output = gok_output_new (OUTPUT_KEYSTRING, cp + offset, SPI_KEY_STRING);
+		}
+		else {
+			/* if not then we are doing a 'replace', handle that via EditableText. */
+			g_warning ("Sorry, string replacement is not yet implemented.");
+		}
+	}
+
+	return last_wordcomplete_output;
+}
