@@ -220,10 +220,12 @@ gok_scanner_find_node (xmlNode *pNode, const xmlChar* name, const xmlChar* targe
 	xmlNode *retval = NULL;
 	int      target_lang_len = strlen ((char *)target_lang);
 	gboolean lang_is_matched = FALSE;	
+	gboolean is_lang_match;
+	gboolean is_perfect_match;
 
 	gok_log_enter();
-	gboolean is_lang_match = FALSE;
-	gboolean is_perfect_match = FALSE;
+	is_lang_match = FALSE;
+	is_perfect_match = FALSE;
 	/* loop through the document looking for the name string */
 	while (pNode != NULL)
 	{
@@ -534,10 +536,10 @@ gboolean gok_scanner_read_access_method (gchar* Filename)
 	gok_scanner_read_options (pDoc, pNodeAccessmethod->xmlChildrenNode, pAccessMethod);
 
 	/* read the rates for this effect */
-	gok_scanner_read_rates (pNodeRoot->xmlChildrenNode, pAccessMethod);
+	gok_scanner_read_rates (pNodeAccessmethod->xmlChildrenNode, pAccessMethod);
 
 	/* read the initializing effects for this access method */
-	pNodeInitialization = gok_keyboard_find_node (pNodeRoot, "initialization");
+	pNodeInitialization = gok_keyboard_find_node (pNodeAccessmethod, "initialization");
 	if (pNodeInitialization != NULL)
 	{
 		pAccessMethod->pInitializationEffects = gok_scanner_read_effects (pNodeInitialization, pAccessMethod);
@@ -546,7 +548,7 @@ gboolean gok_scanner_read_access_method (gchar* Filename)
 	pStateLast = NULL;
 
 	/* get all the handler states */
-	pNodeState = gok_keyboard_find_node (pNodeRoot, "state");
+	pNodeState = gok_keyboard_find_node (pNodeAccessmethod, "state");
 	while (pNodeState != NULL)
 	{
 		/* is this a state? */
@@ -2147,6 +2149,7 @@ void gok_scanner_on_key_enter (GokKey* pKey)
 {
 	m_pKeyEntered = pKey;
 
+	gok_main_raise_window ();
 	gok_scanner_timer_start_dwell();
 		
 	gok_scanner_perform_effects (m_pEffectsOnKeyEnter);
@@ -2416,6 +2419,7 @@ gboolean gok_scanner_on_timer5 (gpointer data)
 **/
 void gok_scanner_on_switch1_down ()
 {
+	gok_main_raise_window ();
 	gok_scanner_perform_effects (m_pEffectsOnSwitch1Down);
 }
 
@@ -2436,6 +2440,7 @@ void gok_scanner_on_switch1_up ()
 **/
 void gok_scanner_on_switch2_down ()
 {
+	gok_main_raise_window ();
 	gok_scanner_perform_effects (m_pEffectsOnSwitch2Down);
 }
 
@@ -2456,6 +2461,7 @@ void gok_scanner_on_switch2_up ()
 **/
 void gok_scanner_on_switch3_down ()
 {
+	gok_main_raise_window ();
 	gok_scanner_perform_effects (m_pEffectsOnSwitch3Down);
 }
 
@@ -2476,6 +2482,7 @@ void gok_scanner_on_switch3_up ()
 **/
 void gok_scanner_on_switch4_down ()
 {
+	gok_main_raise_window ();
 	gok_scanner_perform_effects (m_pEffectsOnSwitch4Down);
 }
 
@@ -2496,6 +2503,7 @@ void gok_scanner_on_switch4_up ()
 **/
 void gok_scanner_on_switch5_down ()
 {
+	gok_main_raise_window ();
 	gok_scanner_perform_effects (m_pEffectsOnSwitch5Down);
 }
 
