@@ -640,6 +640,8 @@ gboolean gok_spy_check_queues(void)
 		switch (en->type)
 		{
 		case GOKSPY_FOCUS_EVENT:
+			/* TODO: remove obsolete focus events from queue since process
+			   accessible is expensive. */
 			gok_log("GOKSPY_FOCUS_EVENT");
 			gok_spy_process_accessible(en->paccessible);
 			break;
@@ -704,13 +706,18 @@ gboolean gok_spy_check_queues(void)
 */ 
 gboolean gok_spy_check_window(AccessibleRole role)
 {
-	/* TODO - improve efficiency here? */
+	/* TODO - improve efficiency here? Also, roles get added and we need
+	   to maintain this...  maybe we need to go about this differently */
 	if ((role == SPI_ROLE_WINDOW) ||
 	    (role == SPI_ROLE_DIALOG) ||
 	    (role == SPI_ROLE_FILE_CHOOSER) ||
 	    (role == SPI_ROLE_FRAME) ||	
 	    (role == SPI_ROLE_DESKTOP_FRAME) ||
-		(role == SPI_ROLE_FONT_CHOOSER))
+		(role == SPI_ROLE_FONT_CHOOSER)	||
+		(role == SPI_ROLE_COLOR_CHOOSER) ||
+		(role == SPI_ROLE_APPLICATION) ||
+		(role == SPI_ROLE_ALERT)
+		)
 	{
 		return TRUE;
 	}
@@ -1272,7 +1279,6 @@ void gok_spy_focus_listener (const AccessibleEvent *event, void *user_data)
 	EventNode* en; 
 	gok_log_enter();
 	gok_spy_accessible_ref(event->source);
-	
 	en = (EventNode*)g_malloc(sizeof(EventNode));
 	en->paccessible = event->source;
 	en->type = GOKSPY_FOCUS_EVENT;
